.TH dlabel 1 "July 2002" "Version 1.31"
.SH NAME
dlabel \- image labeling tool for character pattern extraction
(v1.31)
.SH SYNOPSIS
.B dlabel
.BR
[\-options]
.BR
.I in_file

.SH DESCRIPTION
dlabel is a powerful tool for separating character patterns and 
complex backgrounds in grayscale images.
dlabel can separate character patterns with various gray levels
and sizes from the images with overlapping backgrounds
as long as there is enough difference of gray values between
the character patterns and the backgrounds.

.PP
dlabel can handle document images with colorful, complex backgrounds,
such as those obtained from book covers, magazines, journals and advertisements.
dlabel can handle not only scanned document images but also
scene images taken by a camera.

.PP
dlabel reads a raw pgm(5) image from
.I in_file,
segments locally\-isochromatic objects such as character patterns,
groups the objects,
and writes each object group to an individual raw pbm(5) file.
The names of the output files are 0000.pbm, 0001.pbm, ..., and 
the files are put into the current directory.
These images are called "decomposed images."
Although dlabel can read a raw ppm(5) image file as well,
the image will be converted to the grayscale image internally.

.PP
A typical usage of dlabel looks like this:
.IP
% dlabel -v -w -ri rimage.pgm -label limage.pgm -decomp in_file.pgm

.PP
The simplified descriptions of the processes in this program
are as shown below.
(See references for details of the processes.)

.TP
1.
The input image is partitioned into small (50x50dots) subimages.
.TP
2.
Local representative gray levels are found in every subimage
by analyzing the gray level histogram.
The input image is segmented according to the gray levels
by using multilevel thresholding.
.TP
3.
The locally segmented objects are merged between neighboring 
subimages according to the difference of the corresponding 
representative gray levels.
.TP
4.
Each group of the merged objects is written to a bitmap file,
regarding the objects are black (foreground) images.

.PP
The method used in this program has following advantageous
properties.

.TP
1.
It can separate light character patterns and dark ones
simultaneously.
.TP
2.
It can extract very thin (>1.5pixels) character strokes.
.TP
3.
It is tolerant of dull image in which edges are not 
very clear.
.TP
4.
It is tolerant of slight shadings of image.
Even if the brightness of a text line changes gradually,
it can extract the pattern of the text line as a single image.
(All character patterns in the same text line are
 expected to appear in a single output file.)

.PP
Although dlabel accepts images of various resolutions, 
300\-400dpi is the typical resolution range 
to deal with small characters used in main text blocks
of books and journals.



.SH OPTIONS
.TP
.B \-s
Apply k\-nearest neighbor smoothing to the input image once.
This option is useful for processing noisy images.
The smoothing algorithm is the same as that used in lpsmooth(1).
The parameter is always k=2.
.TP
.B \-w
Use weighted histogram when dlabel finds local representative gray levels.
This option is very useful for extraction of thin character strokes.
Users are strongly recommended to specify this option.
.TP
.B \-ri \fIfile\fP
Write the locally reduced image to 
.I file
in pgm(5) format.
.TP
.B \-label \fIfile\fP
Write the label image to 
.I file
in pgm(5) format.
The label image corrupts if there are more than 256 labels in total.
.TP
.B \-decomp
Create the decomposed images in pbm(5) format.
The files will be put into the current directory.
.TP
.B \-verbose, \-v
Show further processing information during its process.


.SH EXIT STATUS
The following exit values are returned:
.PD 0
.TP 10
.B 0
Successful completion.
.TP
.B > 0
An error occurred.
.PD

.SH SEE ALSO
pbm(5), pgm(5), ppm(5), lpsmooth(1)

.SH NOTES
This program can separate character patterns and background images.
However, it cannot find which images correspond to character patterns
and which to backgrounds.

.PP
This program is not very tolerant of the moire noise caused by
the interference between the halftone image and the sampling grid
at scanning.
This program may not work well for very noisy images.
Some appropriate filters will be required as preprocessings.

.PP
This program was designed so that it would not oversegment character 
patterns or text line patterns as far as possible.
However, background images are often oversegmented greatly.
The program may produce a lot of output files which consist of
oversegmented pieces of image.
(The number of the output files has been greatly
reduced compared to the predecessor v1.21.)

.PP
Unfortunately,
this program may not work well for some images in which
objects (including characters) with different gray levels
appear densely; 
such as bookshelf images taken by a camera.


.SH REFERENCES

Hideaki Goto and Hirotomo Aso,
"Character Pattern Extraction Based on Local Multilevel Thresholding and
Region Growing,"
Proc. 15th Int. Conf. Patt. Recogn. (ICPR2000), 2000.

(dlabel is based on the method proposed in this paper.
However, some processes
and the internal parameters 
are slightly different from those described in the paper.)

.PP
Hideaki Goto, Masatsugu Hirayama and Hirotomo Aso, 
"Character pattern extraction from grayscale document images
using local multilevel thresholding,"
Trans. IEICE (D\-II), Vol.J82\-D\-II, No.11, pp.2188\-2192, 1999.
(in Japanese)

.SH AUTHOR
Copyright (C) 1999-2002 Hideaki Goto
.\" Permission to use, copy, modify, and distribute this software and
.\" its documentation for any purpose is hereby granted without fee,
.\" provided that (i) the above copyright notice and this permission
.\" notice appear in all copies and in supporting documentation, (ii)
.\" the name of the author, Hideaki Goto, may not be used in any
.\" advertising or otherwise to promote the sale, use or other
.\" dealings in this software without prior written authorization
.\" from the author, (iii) this software may not be used for
.\" commercial products without prior written permission from the
.\" author, and (iv) the notice of modification is specified in cases
.\" where modified copies of this software are distributed.
.\"
.\" THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
.\" THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE CAUSED BY THIS
.\" SOFTWARE.

E\-Mail: hgot@rd.isc.tohoku.ac.jp, hgot@aso.ecei.tohoku.ac.jp
