/*----------------------------------------------------------------------
    Functions for image edge compensation           edgecomp.cxx
        Written by  H.Goto, Jun 1999
        Modified by H.Goto, Sep 1999
        Modified by H.Goto, Oct 1999
        Modified by H.Goto, Jun 2002
----------------------------------------------------------------------*/

/*--------------------------------------------------------------------
  Copyright (C) 1999-2002  Hideaki Goto

        All Rights Reserved

  Permission to use, copy, modify, and distribute this software and
  its documentation for any purpose is hereby granted without fee,
  provided that (i) the above copyright notice and this permission
  notice appear in all copies and in supporting documentation, (ii)
  the name of the author, Hideaki Goto, may not be used in any
  advertising or otherwise to promote the sale, use or other
  dealings in this software without prior written authorization
  from the author, (iii) this software may not be used for
  commercial products without prior written permission from the
  author, and (iv) the notice of modification is specified in cases
  where modified copies of this software are distributed.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
  THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE CAUSED BY THIS
  SOFTWARE.
--------------------------------------------------------------------*/

#include        <stdio.h>
#include        <stdlib.h>
#include        <string.h>
#include        <math.h>

#include	"utypes.h"

#include	"siplib.h"
#include	"edgecomp.h"


/*------------------------------------------------------
	Edge detection
------------------------------------------------------*/

static int detect_edge(int *lbuf,int x0,int width,int *xs,int *xe,int maxflatlen){
	int	x,l,d,udflag,lv,diff;
	d = lbuf[x0];
	for ( x=x0 ; x < width && d == lbuf[x] ; x++ ) ;	/* find 1st edge */
	if ( lbuf[x] > d )  udflag = 1;  else  udflag = -1;
	d = lbuf[x];
	*xs = (*xe = x) -1;
	for ( l=1, lv=2, x++ ; x<width ; x++,l++ ){
		diff = udflag * (lbuf[x] - d);
		if ( diff > 0 ){
			l = 0;
			lv++;
			d = lbuf[x];
			*xe = x;
		}
		if ( diff < 0 )  break;
		if ( l > maxflatlen )  break;	/* longest flat point */
	}
	return(lv);
}




/*------------------------------------------------------
	Edge compensation (single image mode)
------------------------------------------------------*/

static int compensate_edge_single(SIPImage *src,SIPImage *dst, \
		int maxflatlen,int mindiff,int maxedgewidth){
	int	x,y,i,xs,xe;
	int	diff1,diff2;
	SIPImage *src0;
	uchar	*p,*dp;
	uchar	**gp;
	int	width,height;
	int	*lbuf;
	width = src->width;
	height = src->height;
	if ( width == 0 || height == 0 )  return(0);
	if ( src->depth != 8 || dst->depth != 8 )  return(-1);
	if ( 0 == (lbuf = new int[ width > height ? width : height ]) )  return(-1);
	if ( 0 == (gp = new uchar * [height]) ){
		delete []lbuf;
		return(-1);
	}
	src0 = src;
	if ( src == dst ){
		if ( 0 == (src0 = sip_CreateImage(width,height,8)) ){
			delete []lbuf;
			delete []gp;
			return(-1);
		}
		for ( y=0 ; y<height ; y++ ){
			memcpy(	(void *)sip_getimgptr(src0,y), \
				(void *)sip_getimgptr(src,y), width);
		}
	}
	for ( y=0 ; y<height ; y++ ){
		gp[y] = (uchar *)sip_getimgptr(dst,y);
	}

	for ( y=0 ; y<height ; y++ ){
		p  = (uchar *)sip_getimgptr(src0,y);
		dp = (uchar *)sip_getimgptr(dst,y);
		for ( x=0 ; x<width ; x++ ){
			lbuf[x] = (int)((uint)p[x]);
			dp[x] = p[x];	/* copy original pixels */
		}

		for ( x=0 ; x<width ; x++ ){
			if ( 2 > detect_edge(lbuf,x,width,&xs,&xe,maxflatlen) ){
				continue;
			}
			if ( mindiff > abs(lbuf[xe] - lbuf[xs]) ){
				x = xe -1;
				continue;
			}
			if ( maxedgewidth < xe - xs ){
				x = xe -1;
				continue;
			}
			for ( i = xs +1 ; i < xe ; i++ ){
				diff1 = abs(lbuf[i] - lbuf[xs]);
				diff2 = abs(lbuf[i] - lbuf[xe]);
				if ( diff1 <= diff2 ){
					dp[i] = dp[xs];
				}
				else{	dp[i] = dp[xe];
				}
			}
			x = xe -1;
		}
	}

	for ( x=0 ; x<width ; x++ ){
		for ( y=0 ; y<height ; y++ ){
			p  = (uchar *)sip_getimgptr(src0,y);
			lbuf[y] = (int)((uint)p[x]);
		}

		for ( y=0 ; y<height ; y++ ){
			if ( 2 > detect_edge(lbuf,y,height,&xs,&xe,maxflatlen) ){
				continue;
			}
			if ( mindiff > abs(lbuf[xe] - lbuf[xs]) ){
				y = xe -1;
				continue;
			}
			if ( maxedgewidth < xe - xs ){
				y = xe -1;
				continue;
			}
			for ( i = xs +1 ; i < xe ; i++ ){
				diff1 = abs(lbuf[i] - lbuf[xs]);
				diff2 = abs(lbuf[i] - lbuf[xe]);
				if ( diff1 <= diff2 ){
					gp[i][x] = gp[xs][x];
				}
				else{	gp[i][x] = gp[xe][x];
				}
			}
			y = xe -1;
		}
	}

	if ( src == dst ){
		sip_DestroyImage(src0);
	}
	delete []gp;
	delete []lbuf;
	return(0);
}




/*------------------------------------------------------
	Edge compensation (main)
------------------------------------------------------*/

int compensate_edge(SIPImage *src,SIPImage *orgsrc,SIPImage *dst, \
		int maxflatlen,int mindiff,int maxedgewidth){
	int	x,y,i,xs,xe;
	int	diff1,diff2;
	SIPImage *src0;
	uchar	*p,*dp,*ip;
	uchar	**gp;
	int	width,height;
	int	*lbuf,*lbuf0;

	if ( orgsrc == 0 || orgsrc == src ){
		return( compensate_edge_single(src,dst, \
			maxflatlen,mindiff,maxedgewidth) );
	}

	width = src->width;
	height = src->height;
	if ( width == 0 || height == 0 )  return(0);
	if ( src->depth != 8 || dst->depth != 8 )  return(-1);
	if ( 0 == (lbuf = new int[ 2 * (width > height ? width : height) ]) )  return(-1);
	lbuf0 = lbuf + (width > height ? width : height);
	if ( 0 == (gp = new uchar * [height]) ){
		delete []lbuf;
		return(-1);
	}
	src0 = src;
	if ( src == dst ){
		if ( 0 == (src0 = sip_CreateImage(width,height,8)) ){
			delete []lbuf;
			delete []gp;
			return(-1);
		}
		for ( y=0 ; y<height ; y++ ){
			memcpy(	(void *)sip_getimgptr(src0,y), \
				(void *)sip_getimgptr(src,y), width);
		}
	}
	for ( y=0 ; y<height ; y++ ){
		gp[y] = (uchar *)sip_getimgptr(dst,y);
	}

	for ( y=0 ; y<height ; y++ ){
		p  = (uchar *)sip_getimgptr(src0,y);
		dp = (uchar *)sip_getimgptr(dst,y);
		ip = (uchar *)sip_getimgptr(orgsrc,y);
		for ( x=0 ; x<width ; x++ ){
			lbuf[x] = (int)((uint)p[x]);
			dp[x] = p[x];	/* copy original pixels */
		}

		for ( x=0 ; x<width ; x++ ){
			if ( 2 > detect_edge(lbuf,x,width,&xs,&xe,maxflatlen) ){
				continue;
			}
			if ( mindiff > abs(lbuf[xe] - lbuf[xs]) ){
				x = xe -1;
				continue;
			}
			if ( maxedgewidth < xe - xs -1 ){
				x = xe -1;
				continue;
			}
			for ( i = xs +1 ; i < xe ; i++ ){
				diff1 = abs((uint)ip[i] - lbuf[xs]);
				diff2 = abs((uint)ip[i] - lbuf[xe]);
				if ( diff1 <= diff2 ){
					dp[i] = dp[xs];
				}
				else{	dp[i] = dp[xe];
				}
			}
			x = xe -1;
		}
	}

	for ( x=0 ; x<width ; x++ ){
		for ( y=0 ; y<height ; y++ ){
			p  = (uchar *)sip_getimgptr(src0,y);
			lbuf[y] = (int)((uint)p[x]);
			ip  = (uchar *)sip_getimgptr(orgsrc,y);
			lbuf0[y] = (int)((uint)ip[x]);
		}

		for ( y=0 ; y<height ; y++ ){
			if ( 2 > detect_edge(lbuf,y,height,&xs,&xe,maxflatlen) ){
				continue;
			}
			if ( mindiff > abs(lbuf[xe] - lbuf[xs]) ){
				y = xe -1;
				continue;
			}
			if ( maxedgewidth < xe - xs ){
				y = xe -1;
				continue;
			}
			for ( i = xs +1 ; i < xe ; i++ ){
				diff1 = abs(lbuf0[i] - lbuf[xs]);
				diff2 = abs(lbuf0[i] - lbuf[xe]);
				if ( diff1 <= diff2 ){
					gp[i][x] = gp[xs][x];
				}
				else{	gp[i][x] = gp[xe][x];
				}
			}
			y = xe -1;
		}
	}

	if ( src == dst ){
		sip_DestroyImage(src0);
	}
	delete []gp;
	delete []lbuf;
	return(0);
}


