/*
	gettime.c, by H.Goto  Apr. 1999
*/

/*--------------------------------------------------------------------
  Copyright (C) 1999  Hideaki Goto

        All Rights Reserved

  Permission to use, copy, modify, and distribute this software and
  its documentation for any purpose is hereby granted without fee,
  provided that (i) the above copyright notice and this permission
  notice appear in all copies and in supporting documentation, (ii)
  the name of the author, Hideaki Goto, may not be used in any
  advertising or otherwise to promote the sale, use or other
  dealings in this software without prior written authorization
  from the author, (iii) this software may not be used for
  commercial products without prior written permission from the
  author, and (iv) the notice of the modification is specified in
  case of that the modified copies of this software are distributed.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
  THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE CAUSED BY THIS
  SOFTWARE.
--------------------------------------------------------------------*/

#include	<stdio.h>

#include	"gettime.h"


#ifdef	HAS_NO_TIMES

/* dummy functions here */

void time_start(void){
	return;
}

int time_get(double *utime,double *stime){
	if ( utime )  *utime = 0;
	if ( stime )  *stime = 0;
	return(0);
}


#else


static int start_t=0;
static struct rusage s_ru;

void time_start(void)
{
	start_t = getrusage(RUSAGE_SELF, &s_ru);
}


int time_get(double *utime,double *stime)
{
	int t2;
	struct rusage ru;
	
	t2 = getrusage(RUSAGE_SELF, &ru);
	if (start_t==-1 || t2==-1){
		fprintf(stderr,"time count failed\n");
		return(-1);
	}
	if ( utime != 0 ){
		if ( ru.ru_utime.tv_sec < 1000 ){
			*utime = (double)((ru.ru_utime.tv_sec * 1000000 \
				+ ru.ru_utime.tv_usec) \
				- (s_ru.ru_utime.tv_sec * 1000000 \
				+ s_ru.ru_utime.tv_usec)) / 1000000;
		}
		else{	*utime = (double)ru.ru_utime.tv_sec;
		}
	}
	if ( stime != 0 ){
		if ( ru.ru_stime.tv_sec < 1000 ){
			*stime = (double)((ru.ru_stime.tv_sec * 1000000 \
				+ ru.ru_stime.tv_usec) \
				- (s_ru.ru_stime.tv_sec * 1000000 \
				+ s_ru.ru_stime.tv_usec)) / 1000000;
		}
		else{	*stime = (double)ru.ru_stime.tv_sec;
		}
	}
	return(0);
}


#endif	/* HAS_NO_TIMES */
